@@ -5,7 +5,7 @@
  | rcube_install.php                                                     |
  |                                                                       |
  | This file is part of the RoundCube Webmail package                    |
- | Copyright (C) 2008, RoundCube Dev. - Switzerland                      |
+ | Copyright (C) 2008-2009, RoundCube Dev. - Switzerland                 |
  | Licensed under the GNU Public License                                 |
  +-----------------------------------------------------------------------+
 
@@ -40,15 +40,8 @@ class rcube_install
     'addrbook_show_images' => 'show_images',
   );
   
-  // these config options are optional or can be set to null
-  var $optional_config = array(
-    'log_driver', 'syslog_id', 'syslog_facility', 'imap_auth_type',
-    'smtp_helo_host', 'smtp_auth_type', 'sendmail_delay', 'double_auth',
-    'language', 'mail_header_delimiter', 'create_default_folders',
-    'quota_zero_as_unlimited', 'spellcheck_uri', 'spellcheck_languages',
-    'http_received_header', 'session_domain', 'mime_magic', 'log_logins',
-    'enable_installer', 'skin_include_php', 'imap_root', 'imap_delimiter',
-    'virtuser_file', 'virtuser_query', 'dont_override');
+  // these config options are required for a working system
+  var $required_config = array('db_dsnw', 'db_table_contactgroups', 'db_table_contactgroupmembers', 'des_key');
   
   /**
    * Constructor
@@ -136,10 +129,10 @@ class rcube_install
    */
   function create_config($which, $force = false)
   {
-    $out = file_get_contents(RCMAIL_CONFIG_DIR . "/{$which}.inc.php.dist");
+    $out = @file_get_contents(RCMAIL_CONFIG_DIR . "/{$which}.inc.php.dist");
     
     if (!$out)
-      return '[Warning: could not read the template file]';
+      return '[Warning: could not read the config template file]';
 
     foreach ($this->config as $prop => $default) {
       $value = (isset($_POST["_$prop"]) || $this->bool_config_props[$prop]) ? $_POST["_$prop"] : $default;
@@ -176,6 +169,18 @@ class rcube_install
       else if ($prop == 'smtp_pass' && !empty($_POST['_smtp_user_u'])) {
         $value = '%p';
       }
+      else if ($prop == 'default_imap_folders'){
+	$value = Array();
+	foreach($this->config['default_imap_folders'] as $_folder){
+	  switch($_folder) {
+	  case 'Drafts': $_folder = $this->config['drafts_mbox']; break;
+	  case 'Sent':   $_folder = $this->config['sent_mbox']; break;
+	  case 'Junk':   $_folder = $this->config['junk_mbox']; break;
+	  case 'Trash':  $_folder = $this->config['trash_mbox']; break;
+          }
+	  if (!in_array($_folder, $value))  $value[] = $_folder;
+        }
+      }
       else if (is_bool($default)) {
         $value = (bool)$value;
       }
@@ -218,7 +223,7 @@ class rcube_install
       return null;
     
     $out = $seen = array();
-    $optional = array_flip($this->optional_config);
+    $required = array_flip($this->required_config);
     
     // iterate over the current configuration
     foreach ($this->config as $prop => $value) {
@@ -234,19 +239,21 @@ class rcube_install
     
     // iterate over default config
     foreach ($defaults as $prop => $value) {
-      if (!$seen[$prop] && !isset($this->config[$prop]) && !isset($optional[$prop]))
+      if (!isset($seen[$prop]) && !isset($this->config[$prop]) && isset($required[$prop]))
         $out['missing'][] = array('prop' => $prop);
     }
-    
+
     // check config dependencies and contradictions
     if ($this->config['enable_spellcheck'] && $this->config['spellcheck_engine'] == 'pspell') {
       if (!extension_loaded('pspell')) {
         $out['dependencies'][] = array('prop' => 'spellcheck_engine',
           'explain' => 'This requires the <tt>pspell</tt> extension which could not be loaded.');
       }
-      if (empty($this->config['spellcheck_languages'])) {
-        $out['dependencies'][] = array('prop' => 'spellcheck_languages',
-          'explain' => 'You should specify the list of languages supported by your local pspell installation.');
+      else if (!empty($this->config['spellcheck_languages'])) {
+        foreach ($this->config['spellcheck_languages'] as $lang => $descr)
+          if (!pspell_new($lang))
+            $out['dependencies'][] = array('prop' => 'spellcheck_languages',
+              'explain' => "You are missing pspell support for language $lang ($descr)");
       }
     }
     
@@ -318,6 +325,44 @@ class rcube_install
     }
   }
   
+  /**
+   * Compare the local database schema with the reference schema
+   * required for this version of RoundCube
+   *
+   * @param boolean True if the schema schould be updated
+   * @return boolean True if the schema is up-to-date, false if not or an error occured
+   */
+  function db_schema_check($DB, $update = false)
+  {
+    if (!$this->configured)
+      return false;
+    
+    // simple ad hand-made db schema
+    $db_schema = array(
+      'users' => array(),
+      'identities' => array(),
+      'contacts' => array(),
+      'contactgroups' => array(),
+      'contactgroupmembers' => array(),
+      'cache' => array(),
+      'messages' => array(),
+      'session' => array(),
+    );
+    
+    $errors = array();
+    
+    // check list of tables
+    $existing_tables = $DB->list_tables();
+
+    foreach ($db_schema as $table => $cols) {
+      $table = !empty($this->config['db_table_'.$table]) ? $this->config['db_table_'.$table] : $table;
+      if (!in_array($table, $existing_tables))
+        $errors[] = "Missing table ".$table;
+      // TODO: check cols and indices
+    }
+    
+    return !empty($errors) ? $errors : false;
+  }
   
   /**
    * Compare the local database schema with the reference schema
@@ -326,7 +371,7 @@ class rcube_install
    * @param boolean True if the schema schould be updated
    * @return boolean True if the schema is up-to-date, false if not or an error occured
    */
-  function db_schema_check($update = false)
+  function mdb2_schema_check($update = false)
   {
     if (!$this->configured)
       return false;
@@ -341,7 +386,8 @@ class rcube_install
       'portability' => true
     );
     
-    $schema =& MDB2_Schema::factory($this->config['db_dsnw'], $options);
+    $dsnw = $this->config['db_dsnw'];
+    $schema = MDB2_Schema::factory($dsnw, $options);
     $schema->db->supported['transactions'] = false;
     
     if (PEAR::isError($schema)) {
@@ -358,10 +404,11 @@ class rcube_install
       }
       
       // load reference schema
-      $dsn = MDB2::parseDSN($this->config['db_dsnw']);
-      $ref_schema = INSTALL_PATH . 'SQL/' . $dsn['phptype'] . '.schema.xml';
+      $dsn_arr = MDB2::parseDSN($this->config['db_dsnw']);
+
+      $ref_schema = INSTALL_PATH . 'SQL/' . $dsn_arr['phptype'] . '.schema.xml';
       
-      if (is_file($ref_schema)) {
+      if (is_readable($ref_schema)) {
         $reference = $schema->parseDatabaseDefinition($ref_schema, false, array(), $schema->options['fail_on_invalid_names']);
         
         if (PEAR::isError($reference)) {
@@ -419,7 +466,7 @@ class rcube_install
     
     foreach ($default_hosts as $key => $name) {
       if (!empty($name))
-        $out[] = is_numeric($key) ? $name : $key;
+        $out[] = rcube_parse_host(is_numeric($key) ? $name : $key);
     }
     
     return $out;
@@ -453,6 +500,20 @@ class rcube_install
     echo Q($name) . ':&nbsp; <span class="fail">NOT OK</span>';
     $this->_showhint($message, $url);
   }
+
+
+  /**
+   * Display an error status for optional settings/features
+   *
+   * @param string Test name
+   * @param string Error message
+   * @param string URL for details
+   */
+  function optfail($name, $message = '', $url = '')
+  {
+    echo Q($name) . ':&nbsp; <span class="na">NOT OK</span>';
+    $this->_showhint($message, $url);
+  }
   
   
   /**
@@ -537,11 +598,11 @@ class rcube_install
     if ($lines = @file($fname, FILE_SKIP_EMPTY_LINES)) {
       $buff = '';
       foreach ($lines as $i => $line) {
-        if (eregi('^--', $line))
+        if (preg_match('/^--/', $line))
           continue;
           
         $buff .= $line . "\n";
-        if (eregi(';$', trim($line))) {
+        if (preg_match('/;$/', trim($line))) {
           $DB->query($buff);
           $buff = '';
           if ($this->get_error())
