@@ -94,12 +94,12 @@ if ($RCI->configured && ($messages = $RCI->check_config())) {
 if ($RCI->configured) {
     $pass = false;
 
-    $dirs[] = $RCI->config['temp_dir'];
+    $dirs[] = $RCI->config['temp_dir'] ? $RCI->config['temp_dir'] : 'temp';
     if($RCI->config['log_driver'] != 'syslog')
-      $dirs[] = $RCI->config['log_dir'];
+      $dirs[] = $RCI->config['log_dir'] ? $RCI->config['log_dir'] : 'logs';
 
     foreach ($dirs as $dir) {
-        $dirpath = $dir{0} == '/' ? $dir : INSTALL_PATH . $dir;
+        $dirpath = $dir[0] == '/' ? $dir : INSTALL_PATH . $dir;
         if (is_writable(realpath($dirpath))) {
             $RCI->pass($dir);
             $pass = true;
@@ -119,7 +119,7 @@ else {
 
 ?>
 
-<h3>Check configured database settings</h3>
+<h3>Check DB config</h3>
 <?php
 
 $db_working = false;
@@ -164,15 +164,13 @@ if ($db_working) {
         echo '<p><input type="submit" name="initdb" value="Initialize database" /></p>';
         $db_working = false;
     }
-  /*
-    else if (!$RCI->db_schema_check($update = !empty($_POST['updatedb']))) {
+    else if ($RCI->db_schema_check($DB, $update = !empty($_POST['updatedb']))) {
         $RCI->fail('DB Schema', "Database schema differs");
-        
-        echo $update ? '<p class="warning">Failed to update the database schema! Please manually execute the SQL statements from the SQL/*.update.sql file on your database</p>' :
-          '<p><input type="submit" name="updatedb" value="Update schema now" /></p>';
+        $updatefile = INSTALL_PATH . 'SQL/' . $DB->db_provider . '.update.sql';
+        echo '<p class="warning">Please manually execute the SQL statements from '.$updatefile.' on your database.<br/>';
+        echo 'See comments in the file and execute queries that are superscribed with the currently installed version number.</p>';
         $db_working = false;
     }
-  */
     else {
         $RCI->pass('DB Schema');
         echo '<br />';
@@ -213,10 +211,10 @@ if ($db_working) {
 
 ?>
 
-<h3>Test SMTP settings</h3>
+<h3>Test SMTP config</h3>
 
 <p>
-Server: <?php echo $RCI->getprop('smtp_server', 'PHP mail()'); ?><br />
+Server: <?php echo rcube_parse_host($RCI->getprop('smtp_server', 'PHP mail()')); ?><br />
 Port: <?php echo $RCI->getprop('smtp_port'); ?><br />
 
 <?php
@@ -246,15 +244,13 @@ $to_field = new html_inputfield(array('name' => '_to', 'id' => 'sendmailto'));
 
 <?php
 
-if (isset($_POST['sendmail']) && !empty($_POST['_from']) && !empty($_POST['_to'])) {
-  
-  require_once 'rcube_smtp.inc';
-  
+if (isset($_POST['sendmail'])) {
+
   echo '<p>Trying to send email...<br />';
-  
+
   if (preg_match('/^' . $RCI->email_pattern . '$/i', trim($_POST['_from'])) &&
       preg_match('/^' . $RCI->email_pattern . '$/i', trim($_POST['_to']))) {
-  
+
     $headers = array(
       'From'    => trim($_POST['_from']),
       'To'      => trim($_POST['_to']),
@@ -267,7 +263,7 @@ if (isset($_POST['sendmail']) && !empty($_POST['_from']) && !empty($_POST['_to']
     // send mail using configured SMTP server
     if ($RCI->getprop('smtp_server')) {
       $CONFIG = $RCI->config;
-      
+
       if (!empty($_POST['_smtp_user'])) {
         $CONFIG['smtp_user'] = $_POST['_smtp_user'];
       }
@@ -275,12 +271,16 @@ if (isset($_POST['sendmail']) && !empty($_POST['_from']) && !empty($_POST['_to']
         $CONFIG['smtp_pass'] = $_POST['_smtp_pass'];
       }
 
-      $mail_object  = new rcube_mail_mime();
+      $mail_object  = new Mail_mime();
       $send_headers = $mail_object->headers($headers);
-      
-      $status = smtp_mail($headers['From'], $headers['To'],
-          ($foo = $mail_object->txtHeaders($send_headers)),
-          $body, $smtp_response);
+
+      $SMTP = new rcube_smtp();
+      $SMTP->connect();
+
+      $status = $SMTP->send_mail($headers['From'], $headers['To'],
+          ($foo = $mail_object->txtHeaders($send_headers)), $body);
+
+      $smtp_response = $SMTP->get_response();
     }
     else {    // use mail()
       $header_str = 'From: ' . $headers['From'];
@@ -326,7 +326,7 @@ if (isset($_POST['sendmail']) && !empty($_POST['_from']) && !empty($_POST['_to']
 <p><input type="submit" name="sendmail" value="Send test mail" /></p>
 
 
-<h3>Test IMAP configuration</h3>
+<h3>Test IMAP config</h3>
 
 <?php
 
@@ -403,7 +403,8 @@ if (isset($_POST['imaptest']) && !empty($_POST['_host']) && !empty($_POST['_user
 <p class="warning">
 
 After completing the installation and the final tests please <b>remove</b> the whole
-installer folder from the document root of the webserver.<br />
+installer folder from the document root of the webserver or make sure that
+enable_installer option in main.inc.php is disabled.<br />
 <br />
 
 These files may expose sensitive configuration data like server passwords and encryption keys
