@@ -5,7 +5,7 @@
  | program/include/rcmail.php                                            |
  |                                                                       |
  | This file is part of the RoundCube Webmail client                     |
- | Copyright (C) 2008-2009, RoundCube Dev. - Switzerland                 |
+ | Copyright (C) 2008-2010, RoundCube Dev. - Switzerland                 |
  | Licensed under the GNU GPL                                            |
  |                                                                       |
  | PURPOSE:                                                              |
@@ -15,7 +15,7 @@
  | Author: Thomas Bruederli <roundcube@gmail.com>                        |
  +-----------------------------------------------------------------------+
 
- $Id: rcmail.php 328 2006-08-30 17:41:21Z thomasb $
+ $Id: rcmail.php 3875 2010-08-06 14:59:34Z alec $
 
 */
 
@@ -28,22 +28,25 @@
  */
 class rcmail
 {
-  static public $main_tasks = array('mail','settings','addressbook','login','logout');
-  
+  static public $main_tasks = array('mail','settings','addressbook','login','logout','utils','dummy');
+
   static private $instance;
-  
+
   public $config;
   public $user;
   public $db;
+  public $session;
+  public $smtp;
   public $imap;
   public $output;
-  public $task = 'mail';
+  public $plugins;
+  public $task;
   public $action = '';
   public $comm_path = './';
-  
+
   private $texts;
-  
-  
+
+
   /**
    * This implements the 'singleton' design pattern
    *
@@ -58,8 +61,8 @@ class rcmail
 
     return self::$instance;
   }
-  
-  
+
+
   /**
    * Private constructor
    */
@@ -67,11 +70,11 @@ class rcmail
   {
     // load configuration
     $this->config = new rcube_config();
-    
+
     register_shutdown_function(array($this, 'shutdown'));
   }
-  
-  
+
+
   /**
    * Initial startup function
    * to register session, create database and imap connections
@@ -80,60 +83,51 @@ class rcmail
    */
   private function startup()
   {
-    $config_all = $this->config->all();
-
     // initialize syslog
     if ($this->config->get('log_driver') == 'syslog') {
       $syslog_id = $this->config->get('syslog_id', 'roundcube');
       $syslog_facility = $this->config->get('syslog_facility', LOG_USER);
       openlog($syslog_id, LOG_ODELAY, $syslog_facility);
     }
-    				
-    // set task and action properties
-    $this->set_task(strip_quotes(get_input_value('_task', RCUBE_INPUT_GPC)));
-    $this->action = asciiwords(get_input_value('_action', RCUBE_INPUT_GPC));
 
     // connect to database
     $GLOBALS['DB'] = $this->get_dbh();
 
-    // use database for storing session data
-    include_once('include/session.inc');
-
-    // set session domain
-    if (!empty($config_all['session_domain'])) {
-      ini_set('session.cookie_domain', $config_all['session_domain']);
-    }
-    // set session garbage collecting time according to session_lifetime
-    if (!empty($config_all['session_lifetime'])) {
-      ini_set('session.gc_maxlifetime', ($config_all['session_lifetime']) * 120);
-    }
-
-    // start PHP session (if not in CLI mode)
-    if ($_SERVER['REMOTE_ADDR'])
-      session_start();
-
-    // set initial session vars
-    if (!isset($_SESSION['auth_time'])) {
-      $_SESSION['auth_time'] = time();
-      $_SESSION['temp'] = true;
-    }
+    // start session
+    $this->session_init();
 
     // create user object
     $this->set_user(new rcube_user($_SESSION['user_id']));
 
+    // configure session (after user config merge!)
+    $this->session_configure();
+
+    // set task and action properties
+    $this->set_task(get_input_value('_task', RCUBE_INPUT_GPC));
+    $this->action = asciiwords(get_input_value('_action', RCUBE_INPUT_GPC));
+
     // reset some session parameters when changing task
-    if ($_SESSION['task'] != $this->task)
-      unset($_SESSION['page']);
+    if ($this->task != 'utils') {
+      if ($this->session && $_SESSION['task'] != $this->task)
+        $this->session->remove('page');
+      // set current task to session
+      $_SESSION['task'] = $this->task;
+    }
 
-    // set current task to session
-    $_SESSION['task'] = $this->task;
+    // init output class
+    if (!empty($_REQUEST['_remote']))
+      $GLOBALS['OUTPUT'] = $this->json_init();
+    else
+      $GLOBALS['OUTPUT'] = $this->load_gui(!empty($_REQUEST['_framed']));
 
-    // create IMAP object
-    if ($this->task == 'mail')
-      $this->imap_init();
+    // create plugin API and load plugins
+    $this->plugins = rcube_plugin_api::get_instance();
+
+    // init plugins
+    $this->plugins->init();
   }
-  
-  
+
+
   /**
    * Setter for application task
    *
@@ -141,17 +135,21 @@ class rcmail
    */
   public function set_task($task)
   {
-    if (!in_array($task, self::$main_tasks))
-      $task = 'mail';
-    
+    $task = asciiwords($task);
+
+    if ($this->user && $this->user->ID)
+      $task = !$task || $task == 'login' ? 'mail' : $task;
+    else
+      $task = 'login';
+
     $this->task = $task;
-    $this->comm_path = $this->url(array('task' => $task));
-    
+    $this->comm_path = $this->url(array('task' => $this->task));
+
     if ($this->output)
-      $this->output->set_env('task', $task);
+      $this->output->set_env('task', $this->task);
   }
-  
-  
+
+
   /**
    * Setter for system user object
    *
@@ -162,22 +160,22 @@ class rcmail
     if (is_object($user)) {
       $this->user = $user;
       $GLOBALS['USER'] = $this->user;
-      
+
       // overwrite config with user preferences
-      $this->config->merge((array)$this->user->get_prefs());
+      $this->config->set_user_prefs((array)$this->user->get_prefs());
     }
-    
+
     $_SESSION['language'] = $this->user->language = $this->language_prop($this->config->get('language', $_SESSION['language']));
 
     // set localization
     setlocale(LC_ALL, $_SESSION['language'] . '.utf8', 'en_US.utf8');
 
-    // workaround for http://bugs.php.net/bug.php?id=18556 
-    if (in_array($_SESSION['language'], array('tr_TR', 'ku', 'az_AZ'))) 
-      setlocale(LC_CTYPE, 'en_US' . '.utf8'); 
+    // workaround for http://bugs.php.net/bug.php?id=18556
+    if (in_array($_SESSION['language'], array('tr_TR', 'ku', 'az_AZ')))
+      setlocale(LC_CTYPE, 'en_US' . '.utf8');
   }
-  
-  
+
+
   /**
    * Check the given string and return a valid language code
    *
@@ -187,17 +185,17 @@ class rcmail
   private function language_prop($lang)
   {
     static $rcube_languages, $rcube_language_aliases;
-    
+
     // user HTTP_ACCEPT_LANGUAGE if no language is specified
     if (empty($lang) || $lang == 'auto') {
        $accept_langs = explode(',', $_SERVER['HTTP_ACCEPT_LANGUAGE']);
        $lang = str_replace('-', '_', $accept_langs[0]);
      }
-     
+
     if (empty($rcube_languages)) {
       @include(INSTALL_PATH . 'program/localization/index.inc');
     }
-    
+
     // check if we have an alias for that language
     if (!isset($rcube_languages[$lang]) && isset($rcube_language_aliases[$lang])) {
       $lang = $rcube_language_aliases[$lang];
@@ -205,7 +203,7 @@ class rcmail
     // try the first two chars
     else if (!isset($rcube_languages[$lang])) {
       $short = substr($lang, 0, 2);
-     
+
       // check if we have an alias for the short language code
       if (!isset($rcube_languages[$short]) && isset($rcube_language_aliases[$short])) {
         $lang = $rcube_language_aliases[$short];
@@ -222,8 +220,8 @@ class rcmail
 
     return $lang;
   }
-  
-  
+
+
   /**
    * Get the current database connection
    *
@@ -242,11 +240,12 @@ class rcmail
 
     return $this->db;
   }
-  
-  
+
+
   /**
    * Return instance of the internal address book class
    *
+   * @param string  Address book identifier
    * @param boolean True if the address book needs to be writeable
    * @return object rcube_contacts Address book object
    */
@@ -255,27 +254,88 @@ class rcmail
     $contacts = null;
     $ldap_config = (array)$this->config->get('ldap_public');
     $abook_type = strtolower($this->config->get('address_book_type'));
-    
-    if ($id && $ldap_config[$id]) {
-      $contacts = new rcube_ldap($ldap_config[$id]);
+
+    $plugin = $this->plugins->exec_hook('addressbook_get', array('id' => $id, 'writeable' => $writeable));
+
+    // plugin returned instance of a rcube_addressbook
+    if ($plugin['instance'] instanceof rcube_addressbook) {
+      $contacts = $plugin['instance'];
+    }
+    else if ($id && $ldap_config[$id]) {
+      $contacts = new rcube_ldap($ldap_config[$id], $this->config->get('ldap_debug'), $this->config->mail_domain($_SESSION['imap_host']));
+    }
+    else if ($id === '0') {
+      $contacts = new rcube_contacts($this->db, $this->user->ID);
     }
     else if ($abook_type == 'ldap') {
       // Use the first writable LDAP address book.
       foreach ($ldap_config as $id => $prop) {
         if (!$writeable || $prop['writable']) {
-          $contacts = new rcube_ldap($prop);
+          $contacts = new rcube_ldap($prop, $this->config->get('ldap_debug'), $this->config->mail_domain($_SESSION['imap_host']));
           break;
         }
       }
     }
-    else {
+    else { // $id == 'sql'
       $contacts = new rcube_contacts($this->db, $this->user->ID);
     }
-    
+
     return $contacts;
   }
-  
-  
+
+
+  /**
+   * Return address books list
+   *
+   * @param boolean True if the address book needs to be writeable
+   * @return array  Address books array
+   */
+  public function get_address_sources($writeable = false)
+  {
+    $abook_type = strtolower($this->config->get('address_book_type'));
+    $ldap_config = $this->config->get('ldap_public');
+    $autocomplete = (array) $this->config->get('autocomplete_addressbooks');
+    $list = array();
+
+    // We are using the DB address book
+    if ($abook_type != 'ldap') {
+      $contacts = new rcube_contacts($this->db, null);
+      $list['0'] = array(
+        'id' => 0,
+        'name' => rcube_label('personaladrbook'),
+        'groups' => $contacts->groups,
+        'readonly' => false,
+        'autocomplete' => in_array('sql', $autocomplete)
+      );
+    }
+
+    if ($ldap_config) {
+      $ldap_config = (array) $ldap_config;
+      foreach ($ldap_config as $id => $prop)
+        $list[$id] = array(
+          'id' => $id,
+          'name' => $prop['name'],
+          'groups' => false,
+          'readonly' => !$prop['writable'],
+          'autocomplete' => in_array('sql', $autocomplete)
+        );
+    }
+
+    $plugin = $this->plugins->exec_hook('addressbooks_list', array('sources' => $list));
+    $list = $plugin['sources'];
+
+    if ($writeable && !empty($list)) {
+      foreach ($list as $idx => $item) {
+        if ($item['readonly']) {
+          unset($list[$idx]);
+        }
+      }
+    }
+
+    return $list;
+  }
+
+
   /**
    * Init output object for GUI and add common scripts.
    * This will instantiate a rcmail_template object and set
@@ -290,16 +350,9 @@ class rcmail
     if (!($this->output instanceof rcube_template))
       $this->output = new rcube_template($this->task, $framed);
 
-    foreach (array('flag_for_deletion','read_when_deleted') as $js_config_var) {
-      $this->output->set_env($js_config_var, $this->config->get($js_config_var));
-    }
-    
     // set keep-alive/check-recent interval
-    if ($keep_alive = $this->config->get('keep_alive')) {
-      // be sure that it's less than session lifetime
-      if ($session_lifetime = $this->config->get('session_lifetime'))
-        $keep_alive = min($keep_alive, $session_lifetime * 60 - 30);
-      $this->output->set_env('keep_alive', max(60, $keep_alive));
+    if ($this->session && ($keep_alive = $this->session->get_keep_alive())) {
+      $this->output->set_env('keep_alive', $keep_alive);
     }
 
     if ($framed) {
@@ -310,29 +363,43 @@ class rcmail
     $this->output->set_env('task', $this->task);
     $this->output->set_env('action', $this->action);
     $this->output->set_env('comm_path', $this->comm_path);
-    $this->output->set_charset($this->config->get('charset', RCMAIL_CHARSET));
+    $this->output->set_charset(RCMAIL_CHARSET);
 
     // add some basic label to client
-    $this->output->add_label('loading');
-    
+    $this->output->add_label('loading', 'servererror');
+
     return $this->output;
   }
-  
-  
+
+
   /**
    * Create an output object for JSON responses
    *
    * @return object rcube_json_output Reference to JSON output object
    */
-  public function init_json()
+  public function json_init()
   {
     if (!($this->output instanceof rcube_json_output))
       $this->output = new rcube_json_output($this->task);
-    
+
     return $this->output;
   }
-  
-  
+
+
+  /**
+   * Create SMTP object and connect to server
+   *
+   * @param boolean True if connection should be established
+   */
+  public function smtp_init($connect = false)
+  {
+    $this->smtp = new rcube_smtp();
+
+    if ($connect)
+      $this->smtp->connect();
+  }
+
+
   /**
    * Create global IMAP object and connect to server
    *
@@ -341,6 +408,10 @@ class rcmail
    */
   public function imap_init($connect = false)
   {
+    // already initialized
+    if (is_object($this->imap))
+      return;
+
     $this->imap = new rcube_imap($this->db);
     $this->imap->debug_level = $this->config->get('debug_level');
     $this->imap->skip_deleted = $this->config->get('skip_deleted');
@@ -352,26 +423,33 @@ class rcmail
 
     // set pagesize from config
     $this->imap->set_pagesize($this->config->get('pagesize', 50));
-    
-    // Setting root and delimiter before iil_Connect can save time detecting them
-    // using NAMESPACE and LIST 
+
+    // Setting root and delimiter before establishing the connection
+    // can save time detecting them using NAMESPACE and LIST
     $options = array(
-      'imap' => $this->config->get('imap_auth_type', 'check'),
-      'delimiter' => isset($_SESSION['imap_delimiter']) ? $_SESSION['imap_delimiter'] : $this->config->get('imap_delimiter'),
+      'auth_method' => $this->config->get('imap_auth_type', 'check'),
+      'delimiter'   => isset($_SESSION['imap_delimiter']) ? $_SESSION['imap_delimiter'] : $this->config->get('imap_delimiter'),
+      'rootdir'     => isset($_SESSION['imap_root']) ? $_SESSION['imap_root'] : $this->config->get('imap_root'),
+      'debug_mode'  => (bool) $this->config->get('imap_debug', 0),
+      'force_caps'  => (bool) $this->config->get('imap_force_caps'),
+      'timeout'     => (int) $this->config->get('imap_timeout', 0),
     );
-    
-    if (isset($_SESSION['imap_root']))
-      $options['rootdir'] = $_SESSION['imap_root'];
-    else if ($imap_root = $this->config->get('imap_root'))
-      $options['rootdir'] = $imap_root;
-    
+
     $this->imap->set_options($options);
-  
+
     // set global object for backward compatibility
     $GLOBALS['IMAP'] = $this->imap;
-    
-    if ($connect)
+
+    $hook = $this->plugins->exec_hook('imap_init', array('fetch_headers' => $this->imap->fetch_add_headers));
+    if ($hook['fetch_headers'])
+      $this->imap->fetch_add_headers = $hook['fetch_headers'];
+
+    // support this parameter for backward compatibility but log warning
+    if ($connect) {
       $this->imap_connect();
+      raise_error(array('code' => 800, 'type' => 'imap', 'file' => __FILE__,
+        'message' => "rcube::imap_init(true) is deprecated, use rcube::imap_connect() instead"), true, false);
+    }
   }
 
 
@@ -382,18 +460,87 @@ class rcmail
    */
   public function imap_connect()
   {
-    $conn = false;
-    
-    if ($_SESSION['imap_host'] && !$this->imap->conn) {
-      if (!($conn = $this->imap->connect($_SESSION['imap_host'], $_SESSION['username'], $this->decrypt_passwd($_SESSION['password']), $_SESSION['imap_port'], $_SESSION['imap_ssl']))) {
+    if (!$this->imap)
+      $this->imap_init();
+
+    if ($_SESSION['imap_host'] && !$this->imap->conn->connected()) {
+      if (!$this->imap->connect($_SESSION['imap_host'], $_SESSION['username'], $this->decrypt($_SESSION['password']), $_SESSION['imap_port'], $_SESSION['imap_ssl'])) {
         if ($this->output)
           $this->output->show_message($this->imap->error_code == -1 ? 'imaperror' : 'sessionerror', 'error');
       }
-
-      $this->set_imap_prop();
+      else {
+        $this->set_imap_prop();
+        return $this->imap->conn;
+      }
     }
 
-    return $conn;
+    return false;
+  }
+
+
+  /**
+   * Create session object and start the session.
+   */
+  public function session_init()
+  {
+    // session started (Installer?)
+    if (session_id())
+      return;
+
+    $lifetime = $this->config->get('session_lifetime', 0) * 60;
+
+    // set session domain
+    if ($domain = $this->config->get('session_domain')) {
+      ini_set('session.cookie_domain', $domain);
+    }
+    // set session garbage collecting time according to session_lifetime
+    if ($lifetime) {
+      ini_set('session.gc_maxlifetime', $lifetime * 2);
+    }
+
+    ini_set('session.cookie_secure', rcube_https_check());
+    ini_set('session.name', 'roundcube_sessid');
+    ini_set('session.use_cookies', 1);
+    ini_set('session.use_only_cookies', 1);
+    ini_set('session.serialize_handler', 'php');
+
+    // use database for storing session data
+    $this->session = new rcube_session($this->get_dbh(), $lifetime);
+
+    $this->session->register_gc_handler('rcmail_temp_gc');
+    if ($this->config->get('enable_caching'))
+      $this->session->register_gc_handler('rcmail_cache_gc');
+
+    // start PHP session (if not in CLI mode)
+    if ($_SERVER['REMOTE_ADDR'])
+      session_start();
+
+    // set initial session vars
+    if (!isset($_SESSION['auth_time'])) {
+      $_SESSION['auth_time'] = time();
+      $_SESSION['temp'] = true;
+    }
+  }
+
+
+  /**
+   * Configure session object internals
+   */
+  public function session_configure()
+  {
+    if (!$this->session)
+      return;
+
+    $lifetime = $this->config->get('session_lifetime', 0) * 60;
+
+    // set keep-alive/check-recent interval
+    if ($keep_alive = $this->config->get('keep_alive')) {
+      // be sure that it's less than session lifetime
+      if ($lifetime)
+        $keep_alive = min($keep_alive, $lifetime - 30);
+      $keep_alive = max(60, $keep_alive);
+      $this->session->set_keep_alive($keep_alive);
+    }
   }
 
 
@@ -428,7 +575,7 @@ class rcmail
       if (!$allowed)
         return false;
       }
-    else if (!empty($config['default_host']) && $host != $config['default_host'])
+    else if (!empty($config['default_host']) && $host != rcube_parse_host($config['default_host']))
       return false;
 
     // parse $host URL
@@ -436,57 +583,74 @@ class rcmail
     if ($a_host['host']) {
       $host = $a_host['host'];
       $imap_ssl = (isset($a_host['scheme']) && in_array($a_host['scheme'], array('ssl','imaps','tls'))) ? $a_host['scheme'] : null;
-      $imap_port = isset($a_host['port']) ? $a_host['port'] : ($imap_ssl ? 993 : $config['default_port']);
+      if(!empty($a_host['port']))
+        $imap_port = $a_host['port'];
+      else if ($imap_ssl && $imap_ssl != 'tls' && (!$config['default_port'] || $config['default_port'] == 143))
+        $imap_port = 993;
     }
-    else
-      $imap_port = $config['default_port'];
 
+    $imap_port = $imap_port ? $imap_port : $config['default_port'];
 
-    /* Modify username with domain if required  
+    /* Modify username with domain if required
        Inspired by Marco <P0L0_notspam_binware.org>
     */
     // Check if we need to add domain
-    if (!empty($config['username_domain']) && !strpos($username, '@')) {
+    if (!empty($config['username_domain']) && strpos($username, '@') === false) {
       if (is_array($config['username_domain']) && isset($config['username_domain'][$host]))
-        $username .= '@'.$config['username_domain'][$host];
+        $username .= '@'.rcube_parse_host($config['username_domain'][$host]);
       else if (is_string($config['username_domain']))
-        $username .= '@'.$config['username_domain'];
+        $username .= '@'.rcube_parse_host($config['username_domain']);
     }
 
-    // try to resolve email address from virtuser table    
-    if (!empty($config['virtuser_file']) && strpos($username, '@'))
-      $username = rcube_user::email2user($username);
+    // try to resolve email address from virtuser table
+    if (strpos($username, '@'))
+      if ($virtuser = rcube_user::email2user($username))
+        $username = $virtuser;
 
     // lowercase username if it's an e-mail address (#1484473)
     if (strpos($username, '@'))
-      $username = rc_strtolower($username);
+      $username = mb_strtolower($username);
 
     // user already registered -> overwrite username
     if ($user = rcube_user::query($username, $host))
       $username = $user->data['username'];
 
+    if (!$this->imap)
+      $this->imap_init();
+
     // exit if IMAP login failed
     if (!($imap_login  = $this->imap->connect($host, $username, $pass, $imap_port, $imap_ssl)))
       return false;
 
+    $this->set_imap_prop();
+
     // user already registered -> update user's record
     if (is_object($user)) {
+      // create default folders on first login
+      if (!$user->data['last_login'] && $config['create_default_folders'])
+        $this->imap->create_default_folders();
       $user->touch();
     }
     // create new system user
     else if ($config['auto_create_user']) {
       if ($created = rcube_user::create($username, $host)) {
         $user = $created;
-
-        // get existing mailboxes (but why?)
-        // $a_mailboxes = $this->imap->list_mailboxes();
+        // create default folders on first login
+        if ($config['create_default_folders'])
+          $this->imap->create_default_folders();
+      }
+      else {
+        raise_error(array(
+          'code' => 600, 'type' => 'php',
+	      'file' => __FILE__, 'line' => __LINE__,
+          'message' => "Failed to create a user record. Maybe aborted by a plugin?"
+          ), true, false);
       }
     }
     else {
       raise_error(array(
-        'code' => 600,
-        'type' => 'php',
-        'file' => RCMAIL_CONFIG_DIR."/main.inc.php",
+        'code' => 600, 'type' => 'php',
+        'file' => __FILE__, 'line' => __LINE__,
         'message' => "Acces denied for new user $username. 'auto_create_user' is disabled"
         ), true, false);
     }
@@ -501,19 +665,15 @@ class rcmail
       $_SESSION['imap_host'] = $host;
       $_SESSION['imap_port'] = $imap_port;
       $_SESSION['imap_ssl']  = $imap_ssl;
-      $_SESSION['password']  = $this->encrypt_passwd($pass);
+      $_SESSION['password']  = $this->encrypt($pass);
       $_SESSION['login_time'] = mktime();
-      
-      if ($_REQUEST['_timezone'] != '_default_')
+
+      if (isset($_REQUEST['_timezone']) && $_REQUEST['_timezone'] != '_default_')
         $_SESSION['timezone'] = floatval($_REQUEST['_timezone']);
 
       // force reloading complete list of subscribed mailboxes
-      $this->set_imap_prop();
       $this->imap->clear_cache('mailboxes');
 
-      if ($config['create_default_folders'])
-          $this->imap->create_default_folders();
-
       return true;
     }
 
@@ -538,7 +698,7 @@ class rcmail
     if (isset($_SESSION['page'])) {
       $this->imap->set_page($_SESSION['page']);
     }
-    
+
     // cache IMAP root and delimiter in session for performance reasons
     $_SESSION['imap_root'] = $this->imap->root_dir;
     $_SESSION['imap_delimiter'] = $this->imap->delimiter;
@@ -554,15 +714,15 @@ class rcmail
   {
     $default_host = $this->config->get('default_host');
     $host = null;
-    
+
     if (is_array($default_host)) {
       $post_host = get_input_value('_host', RCUBE_INPUT_POST);
-      
+
       // direct match in default_host array
       if ($default_host[$post_host] || in_array($post_host, array_values($default_host))) {
         $host = $post_host;
       }
-      
+
       // try to select host by mail domain
       list($user, $domain) = explode('@', get_input_value('_user', RCUBE_INPUT_POST));
       if (!empty($domain)) {
@@ -583,7 +743,7 @@ class rcmail
       $host = get_input_value('_host', RCUBE_INPUT_POST);
     }
     else
-      $host = $default_host;
+      $host = rcube_parse_host($default_host);
 
     return $host;
   }
@@ -595,36 +755,28 @@ class rcmail
    * @param mixed Named parameters array or label name
    * @return string Localized text
    */
-  public function gettext($attrib)
+  public function gettext($attrib, $domain=null)
   {
     // load localization files if not done yet
     if (empty($this->texts))
       $this->load_language();
-    
+
     // extract attributes
     if (is_string($attrib))
       $attrib = array('name' => $attrib);
 
     $nr = is_numeric($attrib['nr']) ? $attrib['nr'] : 1;
-    $vars = isset($attrib['vars']) ? $attrib['vars'] : '';
-
-    $command_name = !empty($attrib['command']) ? $attrib['command'] : NULL;
-    $alias = $attrib['name'] ? $attrib['name'] : ($command_name && $command_label_map[$command_name] ? $command_label_map[$command_name] : '');
+    $name = $attrib['name'] ? $attrib['name'] : '';
 
+    // check for text with domain
+    if ($domain && ($text_item = $this->texts[$domain.'.'.$name]))
+      ;
     // text does not exist
-    if (!($text_item = $this->texts[$alias])) {
-      /*
-      raise_error(array(
-        'code' => 500,
-        'type' => 'php',
-        'line' => __LINE__,
-        'file' => __FILE__,
-        'message' => "Missing localized text for '$alias' in '$sess_user_lang'"), TRUE, FALSE);
-      */
-      return "[$alias]";
+    else if (!($text_item = $this->texts[$name])) {
+      return "[$name]";
     }
 
-    // make text item array 
+    // make text item array
     $a_text_item = is_array($text_item) ? $text_item : array('single' => $text_item);
 
     // decide which text to use
@@ -651,12 +803,9 @@ class rcmail
     // replace vars in text
     if (is_array($attrib['vars'])) {
       foreach ($attrib['vars'] as $var_key => $var_value)
-        $a_replace_vars[$var_key{0}=='$' ? substr($var_key, 1) : $var_key] = $var_value;
+        $text = str_replace($var_key[0]!='$' ? '$'.$var_key : $var_key, $var_value, $text);
     }
 
-    if ($a_replace_vars)
-      $text = preg_replace('/\$\{?([_a-z]{1}[_a-z0-9]*)\}?/ei', '$a_replace_vars["\1"]', $text);
-
     // format output
     if (($attrib['uppercase'] && strtolower($attrib['uppercase']=='first')) || $attrib['ucfirst'])
       return ucfirst($text);
@@ -674,10 +823,10 @@ class rcmail
    *
    * @param string Language ID
    */
-  public function load_language($lang = null)
+  public function load_language($lang = null, $add = array())
   {
     $lang = $this->language_prop(($lang ? $lang : $_SESSION['language']));
-    
+
     // load localized texts
     if (empty($this->texts) || $lang != $_SESSION['language']) {
       $this->texts = array();
@@ -701,9 +850,13 @@ class rcmail
         if (is_array($messages))
           $this->texts = array_merge($this->texts, $messages);
       }
-      
+
       $_SESSION['language'] = $lang;
     }
+
+    // append additional texts (from plugin)
+    if (is_array($add) && !empty($add))
+      $this->texts += $add;
   }
 
 
@@ -725,7 +878,7 @@ class rcmail
             continue;
 
           if ($label = $rcube_languages[$name])
-            $sa_languages[$name] = $label ? $label : $name;
+            $sa_languages[$name] = $label;
         }
         closedir($dh);
       }
@@ -742,8 +895,6 @@ class rcmail
    */
   function authenticate_session()
   {
-    global $SESS_CLIENT_IP, $SESS_CHANGED;
-
     // advanced session authentication
     if ($this->config->get('double_auth')) {
       $now = time();
@@ -758,12 +909,13 @@ class rcmail
       }
     }
     else {
-      $valid = $this->config->get('ip_check') ? $_SERVER['REMOTE_ADDR'] == $SESS_CLIENT_IP : true;
+      $valid = $this->config->get('ip_check') ? $_SERVER['REMOTE_ADDR'] == $this->session->get_ip() : true;
     }
 
     // check session filetime
     $lifetime = $this->config->get('session_lifetime');
-    if (!empty($lifetime) && isset($SESS_CHANGED) && $SESS_CHANGED + $lifetime*60 < time()) {
+    $sess_ts = $this->session->get_ts();
+    if (!empty($lifetime) && !empty($sess_ts) && $sess_ts + $lifetime*60 < time()) {
       $valid = false;
     }
 
@@ -776,6 +928,9 @@ class rcmail
    */
   public function kill_session()
   {
+    $this->plugins->exec_hook('session_destroy');
+
+    $this->session->remove();
     $_SESSION = array('language' => $this->user->language, 'auth_time' => time(), 'temp' => true);
     rcmail::setcookie('sessauth', '-del-', time() - 60);
     $this->user->reset();
@@ -788,13 +943,13 @@ class rcmail
   public function logout_actions()
   {
     $config = $this->config->all();
-    
-    // on logout action we're not connected to imap server  
+
+    // on logout action we're not connected to imap server
     if (($config['logout_purge'] && !empty($config['trash_mbox'])) || $config['logout_expunge']) {
       if (!$this->authenticate_session())
         return;
 
-      $this->imap_init(true);
+      $this->imap_connect();
     }
 
     if ($config['logout_purge'] && !empty($config['trash_mbox'])) {
@@ -813,10 +968,11 @@ class rcmail
    */
   public function shutdown()
   {
-    if (is_object($this->imap)) {
+    if (is_object($this->imap))
       $this->imap->close();
-      $this->imap->write_cache();
-    }
+
+    if (is_object($this->smtp))
+      $this->smtp->disconnect();
 
     if (is_object($this->contacts))
       $this->contacts->close();
@@ -824,9 +980,52 @@ class rcmail
     // before closing the database connection, write session data
     if ($_SERVER['REMOTE_ADDR'])
       session_write_close();
+
+    // write performance stats to logs/console
+    if ($this->config->get('devel_mode')) {
+      if (function_exists('memory_get_usage'))
+        $mem = show_bytes(memory_get_usage());
+      if (function_exists('memory_get_peak_usage'))
+        $mem .= '/'.show_bytes(memory_get_peak_usage());
+
+      $log = $this->task . ($this->action ? '/'.$this->action : '') . ($mem ? " [$mem]" : '');
+      if (defined('RCMAIL_START'))
+        rcube_print_time(RCMAIL_START, $log);
+      else
+        console($log);
+    }
   }
-  
-  
+
+
+  /**
+   * Generate a unique token to be used in a form request
+   *
+   * @return string The request token
+   */
+  public function get_request_token()
+  {
+    $key = $this->task;
+
+    if (!$_SESSION['request_tokens'][$key])
+      $_SESSION['request_tokens'][$key] = md5(uniqid($key . mt_rand(), true));
+
+    return $_SESSION['request_tokens'][$key];
+  }
+
+
+  /**
+   * Check if the current request contains a valid token
+   *
+   * @param int Request method
+   * @return boolean True if request token is valid false if not
+   */
+  public function check_request($mode = RCUBE_INPUT_POST)
+  {
+    $token = get_input_value('_token', $mode);
+    return !empty($token) && $_SESSION['request_tokens'][$this->task] == $token;
+  }
+
+
   /**
    * Create unique authorization hash
    *
@@ -848,64 +1047,110 @@ class rcmail
       return md5($auth_string);
   }
 
+
   /**
-   * Encrypt IMAP password using DES encryption
+   * Encrypt using 3DES
    *
-   * @param string Password to encrypt
-   * @return string Encryprted string
+   * @param string $clear clear text input
+   * @param string $key encryption key to retrieve from the configuration, defaults to 'des_key'
+   * @param boolean $base64 whether or not to base64_encode() the result before returning
+   *
+   * @return string encrypted text
    */
-  public function encrypt_passwd($pass)
+  public function encrypt($clear, $key = 'des_key', $base64 = true)
   {
-    if (function_exists('mcrypt_module_open') && ($td = mcrypt_module_open(MCRYPT_TripleDES, "", MCRYPT_MODE_ECB, ""))) {
+    if (!$clear)
+      return '';
+    /*-
+     * Add a single canary byte to the end of the clear text, which
+     * will help find out how much of padding will need to be removed
+     * upon decryption; see http://php.net/mcrypt_generic#68082
+     */
+    $clear = pack("a*H2", $clear, "80");
+
+    if (function_exists('mcrypt_module_open') &&
+        ($td = mcrypt_module_open(MCRYPT_TripleDES, "", MCRYPT_MODE_CBC, "")))
+    {
       $iv = mcrypt_create_iv(mcrypt_enc_get_iv_size($td), MCRYPT_RAND);
-      mcrypt_generic_init($td, $this->config->get_des_key(), $iv);
-      $cypher = mcrypt_generic($td, $pass);
+      mcrypt_generic_init($td, $this->config->get_crypto_key($key), $iv);
+      $cipher = $iv . mcrypt_generic($td, $clear);
       mcrypt_generic_deinit($td);
       mcrypt_module_close($td);
     }
-    else if (function_exists('des')) {
-      $cypher = des($this->config->get_des_key(), $pass, 1, 0, NULL);
-    }
     else {
-      $cypher = $pass;
+      @include_once('lib/des.inc');
 
-      raise_error(array(
-        'code' => 500,
-        'type' => 'php',
-        'file' => __FILE__,
-        'message' => "Could not convert encrypt password. Make sure Mcrypt is installed or lib/des.inc is available"
-        ), true, false);
+      if (function_exists('des')) {
+        $des_iv_size = 8;
+        $iv = '';
+        for ($i = 0; $i < $des_iv_size; $i++)
+          $iv .= sprintf("%c", mt_rand(0, 255));
+        $cipher = $iv . des($this->config->get_crypto_key($key), $clear, 1, 1, $iv);
+      }
+      else {
+        raise_error(array(
+          'code' => 500, 'type' => 'php',
+          'file' => __FILE__, 'line' => __LINE__,
+          'message' => "Could not perform encryption; make sure Mcrypt is installed or lib/des.inc is available"
+        ), true, true);
+      }
     }
 
-    return base64_encode($cypher);
+    return $base64 ? base64_encode($cipher) : $cipher;
   }
 
-
   /**
-   * Decrypt IMAP password using DES encryption
+   * Decrypt 3DES-encrypted string
    *
-   * @param string Encrypted password
-   * @return string Plain password
+   * @param string $cipher encrypted text
+   * @param string $key encryption key to retrieve from the configuration, defaults to 'des_key'
+   * @param boolean $base64 whether or not input is base64-encoded
+   *
+   * @return string decrypted text
    */
-  public function decrypt_passwd($cypher)
+  public function decrypt($cipher, $key = 'des_key', $base64 = true)
   {
-    if (function_exists('mcrypt_module_open') && ($td = mcrypt_module_open(MCRYPT_TripleDES, "", MCRYPT_MODE_ECB, ""))) {
-      $iv = mcrypt_create_iv(mcrypt_enc_get_iv_size($td), MCRYPT_RAND);
-      mcrypt_generic_init($td, $this->config->get_des_key(), $iv);
-      $pass = mdecrypt_generic($td, base64_decode($cypher));
+    if (!$cipher)
+      return '';
+
+    $cipher = $base64 ? base64_decode($cipher) : $cipher;
+
+    if (function_exists('mcrypt_module_open') &&
+        ($td = mcrypt_module_open(MCRYPT_TripleDES, "", MCRYPT_MODE_CBC, "")))
+    {
+      $iv = substr($cipher, 0, mcrypt_enc_get_iv_size($td));
+      $cipher = substr($cipher, mcrypt_enc_get_iv_size($td));
+      mcrypt_generic_init($td, $this->config->get_crypto_key($key), $iv);
+      $clear = mdecrypt_generic($td, $cipher);
       mcrypt_generic_deinit($td);
       mcrypt_module_close($td);
     }
-    else if (function_exists('des')) {
-      $pass = des($this->config->get_des_key(), base64_decode($cypher), 0, 0, NULL);
-    }
     else {
-      $pass = base64_decode($cypher);
+      @include_once('lib/des.inc');
+
+      if (function_exists('des')) {
+        $des_iv_size = 8;
+        $iv = substr($cipher, 0, $des_iv_size);
+        $cipher = substr($cipher, $des_iv_size);
+        $clear = des($this->config->get_crypto_key($key), $cipher, 0, 1, $iv);
+      }
+      else {
+        raise_error(array(
+          'code' => 500, 'type' => 'php',
+          'file' => __FILE__, 'line' => __LINE__,
+          'message' => "Could not perform decryption; make sure Mcrypt is installed or lib/des.inc is available"
+        ), true, true);
+      }
     }
 
-    return preg_replace('/\x00/', '', $pass);
-  }
+    /*-
+     * Trim PHP's padding and the canary byte; see note in
+     * rcmail::encrypt() and http://php.net/mcrypt_generic#68082
+     */
+    $clear = substr(rtrim($clear, "\0"), 0, -1);
 
+    return $clear;
+  }
 
   /**
    * Build a valid URL to this instance of RoundCube
@@ -918,17 +1163,16 @@ class rcmail
     if (!is_array($p))
       $p = array('_action' => @func_get_arg(0));
 
-    if (!$p['task'] || !in_array($p['task'], rcmail::$main_tasks))
-      $p['task'] = $this->task;
-
-    $p['_task'] = $p['task'];
+    $task = $p['_task'] ? $p['_task'] : ($p['task'] ? $p['task'] : $this->task);
+    $p['_task'] = $task;
     unset($p['task']);
 
     $url = './';
     $delm = '?';
-    foreach (array_reverse($p) as $par => $val)
+    foreach (array_reverse($p) as $key => $val)
     {
       if (!empty($val)) {
+        $par = $key[0] == '_' ? $key : '_'.$key;
         $url .= $delm.urlencode($par).'='.urlencode($val);
         $delm = '&';
       }
@@ -946,9 +1190,13 @@ class rcmail
    */
   public static function setcookie($name, $value, $exp = 0)
   {
+    if (headers_sent())
+      return;
+
     $cookie = session_get_cookie_params();
+
     setcookie($name, $value, $exp, $cookie['path'], $cookie['domain'],
-      ($_SERVER['HTTPS'] && ($_SERVER['HTTPS'] != 'off')));
+      rcube_https_check(), true);
   }
 }
 
